@@ -87,6 +87,11 @@ class ChangePerspective(NewCredPerspective):
 
         if not files:
             log.msg("No files listed in change... bit strange, but not fatal.")
+
+        if changedict.has_key('links'):
+            log.msg("Found links: "+repr(changedict['links']))
+            del changedict['links']
+
         d = self.master.addChange(**changedict)
         # since this is a remote method, we can't return a Change instance, so
         # this just sets the return value to None:
@@ -113,7 +118,7 @@ class PBChangeSource(config.ReconfigurableServiceMixin, base.ChangeSource):
             d += " (prefix '%s')" % self.prefix
         return d
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def reconfigService(self, new_config):
         # calculate the new port
         port = self.port
@@ -122,17 +127,11 @@ class PBChangeSource(config.ReconfigurableServiceMixin, base.ChangeSource):
 
         # and, if it's changed, re-register
         if port != self.registered_port:
-            wfd = defer.waitForDeferred(
-                self._unregister())
-            yield wfd
-            wfd.getResult()
+            yield self._unregister()
             self._register(port)
 
-        wfd = defer.waitForDeferred(
-            config.ReconfigurableServiceMixin.reconfigService(self,
-                                                    new_config))
-        yield wfd
-        wfd.getResult()
+        yield config.ReconfigurableServiceMixin.reconfigService(
+                self, new_config)
 
     def stopService(self):
         d = defer.maybeDeferred(base.ChangeSource.stopService, self)
