@@ -33,6 +33,15 @@ def patch_bug4881():
         from buildbot.monkeypatches import bug4881
         bug4881.patch()
 
+def patch_bug4520():
+    # this bug was patched in twisted-11.1.0, and only affects py26 and up
+    import sys
+    py_26 = (sys.version_info[0] > 2 or
+            (sys.version_info[0] == 2 and sys.version_info[1] >= 6))
+    if twisted.version < versions.Version('twisted', 11, 1, 0) and py_26:
+        from buildbot.monkeypatches import bug4520
+        bug4520.patch()
+
 def patch_bug5079():
     # this bug will hopefully be patched in Twisted-12.0.0; it was probably
     # present in Twisted-8.x.0, but the patch doesn't work
@@ -53,8 +62,15 @@ def patch_sqlalchemy2189():
         from buildbot.monkeypatches import sqlalchemy2189
         sqlalchemy2189.patch()
 
-def patch_all():
+def patch_all(for_tests=False):
     patch_bug4881()
+    patch_bug4520()
     patch_bug5079()
     patch_sqlalchemy2364()
     patch_sqlalchemy2189()
+
+    if for_tests:
+        from buildbot.monkeypatches import servicechecks
+        servicechecks.patch_servicechecks()
+        from buildbot.monkeypatches import testcase_patch
+        testcase_patch.patch_testcase_patch()
