@@ -358,52 +358,42 @@ class StaticHTML(HtmlResource):
         template = request.site.buildbot_service.templates.get_template("empty.html")
         return template.render(**cxt)
 
-# DirectoryLister isn't available in Twisted-2.5.0, and isn't compatible with what
-# we need until 9.0.0, so we just skip this particular feature.
-have_DirectoryLister = False
-if hasattr(static, 'DirectoryLister'):
-    have_DirectoryLister = True
-    class DirectoryLister(static.DirectoryLister, ContextMixin):
-        """This variant of the static.DirectoryLister uses a template
-        for rendering."""
+class DirectoryLister(static.DirectoryLister, ContextMixin):
+    """This variant of the static.DirectoryLister uses a template
+    for rendering."""
 
-        pageTitle = 'BuildBot'
+    pageTitle = 'BuildBot'
 
-        def render(self, request):
-            cxt = self.getContext(request)
+    def render(self, request):
+        cxt = self.getContext(request)
 
-            if self.dirs is None:
-                directory = os.listdir(self.path)
-                directory.sort()
-            else:
-                directory = self.dirs
+        if self.dirs is None:
+            directory = os.listdir(self.path)
+            directory.sort()
+        else:
+            directory = self.dirs
 
-            dirs, files = self._getFilesAndDirectories(directory)
+        dirs, files = self._getFilesAndDirectories(directory)
 
-            cxt['path'] = cgi.escape(urllib.unquote(request.uri))
-            cxt['directories'] = dirs
-            cxt['files'] = files
-            template = request.site.buildbot_service.templates.get_template("directory.html")
-            data = template.render(**cxt)
-            if isinstance(data, unicode):
-                data = data.encode("utf-8")
-            return data
+        cxt['path'] = cgi.escape(urllib.unquote(request.uri))
+        cxt['directories'] = dirs
+        cxt['files'] = files
+        template = request.site.buildbot_service.templates.get_template("directory.html")
+        data = template.render(**cxt)
+        if isinstance(data, unicode):
+            data = data.encode("utf-8")
+        return data
 
 class StaticFile(static.File):
     """This class adds support for templated directory
     views."""
 
     def directoryListing(self):
-        if have_DirectoryLister:
-            return DirectoryLister(self.path,
-                                   self.listNames(),
-                                   self.contentTypes,
-                                   self.contentEncodings,
-                                   self.defaultType)
-        else:
-            return static.Data("""
-   Directory Listings require Twisted-9.0.0 or later
-                """, "text/plain")
+        return DirectoryLister(self.path,
+                                self.listNames(),
+                                self.contentTypes,
+                                self.contentEncodings,
+                                self.defaultType)
 
 
 MINUTE = 60
@@ -442,9 +432,21 @@ class BuildLineMixin:
         builder_name = build.getBuilder().getName()
         results = build.getResults()
         text = build.getText()
-        rev = str(build.getProperty("got_revision", "??"))
+        all_got_revision = build.getAllGotRevisions()
         css_class = css_classes.get(results, "")
-        repo = build.getSourceStamp().repository
+        ss_list = build.getSourceStamps()
+        if ss_list:
+            repo = ss_list[0].repository
+            if all_got_revision:
+                if len(ss_list) == 1:
+                    rev = all_got_revision.get(ss_list[0].codebase, "??")
+                else:
+                    rev = "multiple rev."
+            else:
+                rev = "??"
+        else:
+            repo = 'unknown, no information in build'
+            rev = 'unknown'
 
         if type(text) == list:
             text = " ".join(text)
@@ -479,7 +481,7 @@ def map_branches(branches):
 # jinja utilities
 
 def createJinjaEnv(revlink=None, changecommentlink=None,
-                     repositories=None, projects=None):
+                     repositories=None, projects=None, jinja_loaders=None):
     ''' Create a jinja environment changecommentlink is used to
         render HTML in the WebStatus and for mail changes
 
@@ -501,10 +503,12 @@ def createJinjaEnv(revlink=None, changecommentlink=None,
     # See http://buildbot.net/trac/ticket/658
     assert not hasattr(sys, "frozen"), 'Frozen config not supported with jinja (yet)'
 
-    default_loader = jinja2.PackageLoader('buildbot.status.web', 'templates')
-    root = os.path.join(os.getcwd(), 'templates')
-    loader = jinja2.ChoiceLoader([jinja2.FileSystemLoader(root),
-                                  default_loader])
+    all_loaders = [jinja2.FileSystemLoader(os.path.join(os.getcwd(), 'templates'))]
+    if jinja_loaders:
+        all_loaders.extend(jinja_loaders)
+    all_loaders.append(jinja2.PackageLoader('buildbot.status.web', 'templates'))
+    loader = jinja2.ChoiceLoader(all_loaders)
+
     env = jinja2.Environment(loader=loader,
                              extensions=['jinja2.ext.i18n'],
                              trim_blocks=True,
@@ -512,6 +516,8 @@ def createJinjaEnv(revlink=None, changecommentlink=None,
 
     env.install_null_translations() # needed until we have a proper i18n backend
 
+    env.tests['mapping'] = lambda obj : isinstance(obj, dict)
+
     env.filters.update(dict(
         urlencode = urllib.quote,
         email = emailfilter,
@@ -786,3 +792,14 @@ class AlmostStrictUndefined(jinja2.StrictUndefined):
         fully as strict as StrictUndefined '''
     def __nonzero__(self):
         return False
+
+_charsetRe = re.compile('charset=([^;]*)', re.I)
+def getRequestCharset(req):
+    """Get the charset for an x-www-form-urlencoded request"""
+    # per http://stackoverflow.com/questions/708915/detecting-the-character-encoding-of-an-http-post-request
+    hdr = req.getHeader('Content-Type')
+    if hdr:
+        mo = _charsetRe.search(hdr)
+        if mo:
+            return mo.group(1).strip()
+    return 'utf-8' # reasonable guess, works for ascii
