@@ -13,32 +13,33 @@
 #
 # Copyright Buildbot Team Members
 
-import os, types
+import os, types, re
 from twisted.python import runtime
-from twisted.internet import reactor 
+from twisted.internet import reactor
 from buildbot.process.buildstep import BuildStep
 from buildbot.process.buildstep import SUCCESS, FAILURE
+from twisted.internet import error
 from twisted.internet.protocol import ProcessProtocol
 
 class MasterShellCommand(BuildStep):
     """
     Run a shell command locally - on the buildmaster.  The shell command
     COMMAND is specified just as for a RemoteShellCommand.  Note that extra
-    logfiles are not sopported.
+    logfiles are not supported.
     """
     name='MasterShellCommand'
     description='Running'
     descriptionDone='Ran'
+    descriptionSuffix = None
+    renderables = [ 'command', 'env', 'description', 'descriptionDone', 'descriptionSuffix' ]
+    haltOnFailure = True
+    flunkOnFailure = True
 
     def __init__(self, command,
-                 description=None, descriptionDone=None,
-                 env=None, path=None, usePTY=0,
+                 description=None, descriptionDone=None, descriptionSuffix=None,
+                 env=None, path=None, usePTY=0, interruptSignal="KILL",
                  **kwargs):
         BuildStep.__init__(self, **kwargs)
-        self.addFactoryArguments(description=description,
-                                 descriptionDone=descriptionDone,
-                                 env=env, path=path, usePTY=usePTY,
-                                 command=command)
 
         self.command=command
         if description:
@@ -49,9 +50,14 @@ class MasterShellCommand(BuildStep):
             self.descriptionDone = descriptionDone
         if isinstance(self.descriptionDone, str):
             self.descriptionDone = [self.descriptionDone]
+        if descriptionSuffix:
+            self.descriptionSuffix = descriptionSuffix
+        if isinstance(self.descriptionSuffix, str):
+            self.descriptionSuffix = [self.descriptionSuffix]
         self.env=env
         self.path=path
         self.usePTY=usePTY
+        self.interruptSignal = interruptSignal
 
     class LocalPP(ProcessProtocol):
         def __init__(self, step):
@@ -64,18 +70,20 @@ class MasterShellCommand(BuildStep):
             self.step.stdio_log.addStderr(data)
 
         def processEnded(self, status_object):
-            self.step.stdio_log.addHeader("exit status %d\n" % status_object.value.exitCode)
+            if status_object.value.exitCode is not None:
+                self.step.stdio_log.addHeader("exit status %d\n" % status_object.value.exitCode)
+            if status_object.value.signal is not None:
+                self.step.stdio_log.addHeader("signal %s\n" % status_object.value.signal)
             self.step.processEnded(status_object)
 
     def start(self):
         # render properties
-        properties = self.build.getProperties()
-        command = properties.render(self.command)
+        command = self.command
         # set up argv
         if type(command) in types.StringTypes:
             if runtime.platformType  == 'win32':
                 argv = os.environ['COMSPEC'].split() # allow %COMSPEC% to have args
-                if '/c' not in argv: argv += ['/c'] 
+                if '/c' not in argv: argv += ['/c']
                 argv += [command]
             else:
                 # for posix, use /bin/sh. for other non-posix, well, doesn't
@@ -84,7 +92,7 @@ class MasterShellCommand(BuildStep):
         else:
             if runtime.platformType  == 'win32':
                 argv = os.environ['COMSPEC'].split() # allow %COMSPEC% to have args
-                if '/c' not in argv: argv += ['/c'] 
+                if '/c' not in argv: argv += ['/c']
                 argv += list(command)
             else:
                 argv = command
@@ -98,22 +106,55 @@ class MasterShellCommand(BuildStep):
         stdio_log.addHeader("** RUNNING ON BUILDMASTER **\n")
         stdio_log.addHeader(" in dir %s\n" % os.getcwd())
         stdio_log.addHeader(" argv: %s\n" % (argv,))
-        self.step_status.setText(list(self.description))
+        self.step_status.setText(self.describe())
 
         if self.env is None:
             env = os.environ
         else:
             assert isinstance(self.env, dict)
             env = self.env
+
+            # do substitution on variable values matching pattern: ${name}
+            p = re.compile('\${([0-9a-zA-Z_]*)}')
+            def subst(match):
+                return os.environ.get(match.group(1), "")
+            newenv = {}
+            for key in env.keys():
+                if env[key] is not None:
+                    newenv[key] = p.sub(subst, env[key])
+            env = newenv
+        stdio_log.addHeader(" env: %r\n" % (env,))
+
         # TODO add a timeout?
-        reactor.spawnProcess(self.LocalPP(self), argv[0], argv, 
+        self.process = reactor.spawnProcess(self.LocalPP(self), argv[0], argv,
                 path=self.path, usePTY=self.usePTY, env=env )
         # (the LocalPP object will call processEnded for us)
 
     def processEnded(self, status_object):
-        if status_object.value.exitCode != 0:
-            self.step_status.setText(["failed (%d)" % status_object.value.exitCode])
+        if status_object.value.signal is not None:
+            self.descriptionDone = ["killed (%s)" % status_object.value.signal]
+            self.step_status.setText(self.describe(done=True))
+            self.finished(FAILURE)
+        elif status_object.value.exitCode != 0:
+            self.descriptionDone = ["failed (%d)" % status_object.value.exitCode]
+            self.step_status.setText(self.describe(done=True))
             self.finished(FAILURE)
         else:
-            self.step_status.setText(list(self.descriptionDone))
+            self.step_status.setText(self.describe(done=True))
             self.finished(SUCCESS)
+
+    def describe(self, done=False):
+        desc = self.descriptionDone if done else self.description
+        if self.descriptionSuffix:
+            desc = desc[:]
+            desc.extend(self.descriptionSuffix)
+        return desc
+
+    def interrupt(self, reason):
+        try:
+            self.process.signalProcess(self.interruptSignal)
+        except KeyError: # Process not started yet
+            pass
+        except error.ProcessExitedAlready:
+            pass
+        BuildStep.interrupt(self, reason)
