@@ -30,8 +30,6 @@ class SetPropertiesFromEnv(buildstep.BuildStep):
 
     def __init__(self, variables, source="SlaveEnvironment", **kwargs):
         buildstep.BuildStep.__init__(self, **kwargs)
-        self.addFactoryArguments(variables = variables,
-                                 source = source)
         self.variables = variables
         self.source = source
 
@@ -43,8 +41,9 @@ class SetPropertiesFromEnv(buildstep.BuildStep):
         fold_to_uppercase = (self.buildslave.slave_system == 'win32')
 
         properties = self.build.getProperties()
-        environ = self.build.slaveEnvironment
+        environ = self.buildslave.slave_environ
         variables = self.variables
+        log = []
         if isinstance(variables, str):
             variables = [self.variables]
         for variable in variables:
@@ -56,6 +55,8 @@ class SetPropertiesFromEnv(buildstep.BuildStep):
                 # note that the property is not uppercased
                 properties.setProperty(variable, value, self.source,
                                        runtime=True)
+                log.append("%s = %r" % (variable, value))
+        self.addCompleteLog("properties", "\n".join(log))
         self.finished(SUCCESS)
 
 class FileExists(buildstep.BuildStep):
@@ -74,7 +75,6 @@ class FileExists(buildstep.BuildStep):
 
     def __init__(self, file, **kwargs):
         buildstep.BuildStep.__init__(self, **kwargs)
-        self.addFactoryArguments(file = file)
         self.file = file
 
     def start(self):
@@ -88,7 +88,7 @@ class FileExists(buildstep.BuildStep):
         d.addErrback(self.failed)
 
     def commandComplete(self, cmd):
-        if cmd.rc != 0:
+        if cmd.didFail():
             self.step_status.setText(["File not found."])
             self.finished(FAILURE)
             return
@@ -115,7 +115,6 @@ class RemoveDirectory(buildstep.BuildStep):
 
     def __init__(self, dir, **kwargs):
         buildstep.BuildStep.__init__(self, **kwargs)
-        self.addFactoryArguments(dir = dir)
         self.dir = dir
 
     def start(self):
@@ -129,7 +128,7 @@ class RemoveDirectory(buildstep.BuildStep):
         d.addErrback(self.failed)
 
     def commandComplete(self, cmd):
-        if cmd.rc != 0:
+        if cmd.didFail():
             self.step_status.setText(["Delete failed."])
             self.finished(FAILURE)
             return
@@ -150,7 +149,6 @@ class MakeDirectory(buildstep.BuildStep):
 
     def __init__(self, dir, **kwargs):
         buildstep.BuildStep.__init__(self, **kwargs)
-        self.addFactoryArguments(dir = dir)
         self.dir = dir
 
     def start(self):
@@ -164,7 +162,7 @@ class MakeDirectory(buildstep.BuildStep):
         d.addErrback(self.failed)
 
     def commandComplete(self, cmd):
-        if cmd.rc != 0:
+        if cmd.didFail():
             self.step_status.setText(["Create failed."])
             self.finished(FAILURE)
             return
