@@ -21,7 +21,7 @@ from twisted.python import log
 from twisted.internet import defer
 
 from buildbot.process import buildstep
-from buildbot.steps.source import Source
+from buildbot.steps.source.base import Source
 from buildbot.interfaces import BuildSlaveTooOldError
 from buildbot.config import ConfigErrors
 
@@ -29,20 +29,17 @@ class SVN(Source):
     """I perform Subversion checkout/update operations."""
 
     name = 'svn'
-    branch_placeholder = '%%BRANCH%%'
 
-    renderables = [ 'repourl', 'baseURL' ]
+    renderables = [ 'repourl' ]
     possible_modes = ('incremental', 'full')
     possible_methods = ('clean', 'fresh', 'clobber', 'copy', 'export', None)
 
-    def __init__(self, repourl=None, baseURL=None, mode='incremental',
-                 method=None, defaultBranch=None, username=None,
+    def __init__(self, repourl=None, mode='incremental',
+                 method=None, username=None,
                  password=None, extra_args=None, keep_on_purge=None,
                  depth=None, **kwargs):
 
         self.repourl = repourl
-        self.baseURL = baseURL
-        self.branch = defaultBranch
         self.username = username
         self.password = password
         self.extra_args = extra_args
@@ -51,35 +48,21 @@ class SVN(Source):
         self.method=method
         self.mode = mode
         Source.__init__(self, **kwargs)
-        self.addFactoryArguments(repourl=repourl,
-                                 baseURL=baseURL,
-                                 mode=mode,
-                                 method=method,
-                                 defaultBranch=defaultBranch,
-                                 password=password,
-                                 username=username,
-                                 extra_args=extra_args,
-                                 keep_on_purge=keep_on_purge,
-                                 depth=depth,
-                                 )
         errors = []
         if self.mode not in self.possible_modes:
             errors.append("mode %s is not one of %s" % (self.mode, self.possible_modes))
         if self.method not in self.possible_methods:
             errors.append("method %s is not one of %s" % (self.method, self.possible_methods))
 
-        if repourl and baseURL:
-            errors.append("you must provide exactly one of repourl and baseURL")
+        if repourl is None:
+            errors.append("you must provide repourl")
 
-        if repourl is None and baseURL is None:
-            errors.append("you must privide at least one of repourl and baseURL")
         if errors:
             raise ConfigErrors(errors)
 
     def startVC(self, branch, revision, patch):
         self.revision = revision
         self.method = self._getMethod()
-        self.repourl = self.getRepoUrl(branch)
         self.stdio_log = self.addLog("stdio")
 
         d = self.checkSvn()
@@ -87,6 +70,7 @@ class SVN(Source):
             if not svnInstalled:
                 raise BuildSlaveTooOldError("SVN is not installed on slave")
             return 0
+        d.addCallback(checkInstall)
 
         if self.mode == 'full':
             d.addCallback(self.full)
@@ -97,95 +81,63 @@ class SVN(Source):
         d.addErrback(self.failed)
         return d
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def full(self, _):
         if self.method == 'clobber':
-            wfd = defer.waitForDeferred(self.clobber())
-            yield wfd
-            wfd.getResult()
+            yield self.clobber()
             return
         elif self.method in ['copy', 'export']:
-            wfd = defer.waitForDeferred(self.copy())
-            yield wfd
-            wfd.getResult()
+            yield self.copy()
             return
 
-        wfd = defer.waitForDeferred(self._sourcedirIsUpdatable())
-        yield wfd
-        updatable = wfd.getResult()
+        updatable = yield self._sourcedirIsUpdatable()
         if not updatable:
             # blow away the old (un-updatable) directory
-            wfd = defer.waitForDeferred(
-                self._rmdir(self.workdir))
-            yield wfd
-            wfd.getResult()
+            yield self._rmdir(self.workdir)
 
             # then do a checkout
             checkout_cmd = ['checkout', self.repourl, '.']
             if self.revision:
                 checkout_cmd.extend(["--revision", str(self.revision)])
-            wfd = defer.waitForDeferred(
-                self._dovccmd(checkout_cmd))
-            yield wfd
-            wfd.getResult()
+            yield self._dovccmd(checkout_cmd)
         elif self.method == 'clean':
-            wfd = defer.waitForDeferred(
-                self.clean())
-            yield wfd
-            wfd.getResult()
+            yield self.clean()
         elif self.method == 'fresh':
-            wfd = defer.waitForDeferred(
-                self.fresh())
-            yield wfd
-            wfd.getResult()
+            yield self.fresh()
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def incremental(self, _):
-        wfd = defer.waitForDeferred(
-            self._sourcedirIsUpdatable())
-        yield wfd
-        updatable = wfd.getResult()
+        updatable = yield self._sourcedirIsUpdatable()
 
         if not updatable:
             # blow away the old (un-updatable) directory
-            wfd = defer.waitForDeferred(
-                self._rmdir(self.workdir))
-            yield wfd
-            wfd.getResult()
+            yield self._rmdir(self.workdir)
 
             # and plan to do a checkout
             command = ['checkout', self.repourl, '.']
         else:
             # otherwise, do an update
             command = ['update']
+
         if self.revision:
             command.extend(['--revision', str(self.revision)])
 
-        wfd = defer.waitForDeferred(
-            self._dovccmd(command))
-        yield wfd
-        wfd.getResult()
+        yield self._dovccmd(command)
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def clobber(self):
         cmd = buildstep.RemoteCommand('rmdir', {'dir': self.workdir,
                                                 'logEnviron': self.logEnviron,})
         cmd.useLog(self.stdio_log, False)
-        wfd = defer.waitForDeferred(
-                self.runCommand(cmd))
-        yield wfd
-        wfd.getResult()
-        if cmd.rc != 0:
+        yield self.runCommand(cmd)
+        if cmd.didFail():
             raise buildstep.BuildStepFailed()
         
         checkout_cmd = ['checkout', self.repourl, '.']
         if self.revision:
             checkout_cmd.extend(["--revision", str(self.revision)])
 
-        wfd = defer.waitForDeferred(
-                self._dovccmd(checkout_cmd))
-        yield wfd
-        wfd.getResult()
+        yield self._dovccmd(checkout_cmd)
 
     def fresh(self):
         d = self.purge(True)
@@ -203,27 +155,21 @@ class SVN(Source):
         d.addCallback(lambda _: self._dovccmd(cmd))
         return d
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def copy(self):
         cmd = buildstep.RemoteCommand('rmdir', {'dir': self.workdir,
                                                 'logEnviron': self.logEnviron,})
         cmd.useLog(self.stdio_log, False)
-        wfd = defer.waitForDeferred(
-                self.runCommand(cmd))
-        yield wfd
-        wfd.getResult()
+        yield self.runCommand(cmd)
 
-        if cmd.rc != 0:
+        if cmd.didFail():
             raise buildstep.BuildStepFailed()
 
         # temporarily set workdir = 'source' and do an incremental checkout
         try:
             old_workdir = self.workdir
             self.workdir = 'source'
-            wfd = defer.waitForDeferred(
-                    self.incremental(None))
-            yield wfd
-            wfd.getResult()
+            yield self.incremental(None)
         except: # finally doesn't work in python-2.4
             self.workdir = old_workdir
             raise
@@ -241,15 +187,12 @@ class SVN(Source):
             export_cmd.extend(['source', self.workdir])
 
             cmd = buildstep.RemoteShellCommand('', export_cmd,
-                    env=self.env, logEnviron=self.logEnviron)
+                    env=self.env, logEnviron=self.logEnviron, timeout=self.timeout)
         cmd.useLog(self.stdio_log, False)
 
-        wfd = defer.waitForDeferred(
-                self.runCommand(cmd))
-        yield wfd
-        wfd.getResult()
+        yield self.runCommand(cmd)
 
-        if cmd.rc != 0:
+        if cmd.didFail():
             raise buildstep.BuildStepFailed()
 
     def finish(self, res):
@@ -261,16 +204,13 @@ class SVN(Source):
         d.addCallbacks(self.finished, self.checkDisconnect)
         return d
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def _rmdir(self, dir):
         cmd = buildstep.RemoteCommand('rmdir',
                 {'dir': dir, 'logEnviron': self.logEnviron })
         cmd.useLog(self.stdio_log, False)
-        wfd = defer.waitForDeferred(
-                self.runCommand(cmd))
-        yield wfd
-        wfd.getResult()
-        if cmd.rc != 0:
+        yield self.runCommand(cmd)
+        if cmd.didFail():
             raise buildstep.BuildStepFailed()
 
     def _dovccmd(self, command, collectStdout=False):
@@ -288,12 +228,13 @@ class SVN(Source):
         cmd = buildstep.RemoteShellCommand(self.workdir, ['svn'] + command,
                                            env=self.env,
                                            logEnviron=self.logEnviron,
+                                           timeout=self.timeout,
                                            collectStdout=collectStdout)
         cmd.useLog(self.stdio_log, False)
         log.msg("Starting SVN command : svn %s" % (" ".join(command), ))
         d = self.runCommand(cmd)
         def evaluateCommand(cmd):
-            if cmd.rc != 0:
+            if cmd.didFail():
                 log.msg("Source step failed while running command %s" % cmd)
                 raise buildstep.BuildStepFailed()
             if collectStdout:
@@ -303,25 +244,6 @@ class SVN(Source):
         d.addCallback(lambda _: evaluateCommand(cmd))
         return d
 
-    def getRepoUrl(self, branch):
-        ''' Compute the svn url that will be passed to the svn remote command '''
-        if self.repourl:
-            return self.repourl
-        else:
-            if branch is None:
-                m = ("The SVN source step belonging to builder '%s' does not know "
-                     "which branch to work with. This means that the change source "
-                     "did not specify a branch and that defaultBranch is None." \
-                     % self.build.builder.name)
-                raise RuntimeError(m)
-
-            computed = self.baseURL
-
-            if self.branch_placeholder in self.baseURL:
-                return computed.replace(self.branch_placeholder, branch)
-            else:
-                return computed + branch
-
     def _getMethod(self):
         if self.method is not None and self.mode != 'incremental':
             return self.method
@@ -330,31 +252,25 @@ class SVN(Source):
         elif self.method is None and self.mode == 'full':
             return 'fresh'
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def _sourcedirIsUpdatable(self):
         # first, perform a stat to ensure that this is really an svn directory
         cmd = buildstep.RemoteCommand('stat', {'file': self.workdir + '/.svn',
                                                'logEnviron': self.logEnviron,})
         cmd.useLog(self.stdio_log, False)
-        wfd = defer.waitForDeferred(
-            self.runCommand(cmd))
-        yield wfd
-        wfd.getResult()
+        yield self.runCommand(cmd)
 
-        if cmd.rc != 0:
-            yield False
+        if cmd.didFail():
+            defer.returnValue(False)
             return
 
         # then run 'svn info' to check that the URL matches our repourl
-        wfd = defer.waitForDeferred(
-            self._dovccmd(['info'], collectStdout=True))
-        yield wfd
-        stdout = wfd.getResult()
+        stdout = yield self._dovccmd(['info'], collectStdout=True)
 
         # extract the URL, handling whitespace carefully so that \r\n works
         # is a line terminator
         mo = re.search('^URL:\s*(.*?)\s*$', stdout, re.M)
-        yield mo and mo.group(1) == self.repourl
+        defer.returnValue(mo and mo.group(1) == self.repourl)
         return
 
     def parseGotRevision(self, _):
@@ -367,6 +283,7 @@ class SVN(Source):
         cmd = buildstep.RemoteShellCommand(svnversion_dir, ['svnversion'],
                                            env=self.env,
                                            logEnviron=self.logEnviron,
+                                           timeout=self.timeout,
                                            collectStdout=True)
         cmd.useLog(self.stdio_log, False)
         d = self.runCommand(cmd)
@@ -383,7 +300,7 @@ class SVN(Source):
                 raise buildstep.BuildStepFailed()
 
             log.msg("Got SVN revision %s" % (revision, ))
-            self.setProperty('got_revision', revision, 'Source')
+            self.updateSourceProperty('got_revision', revision)
             return 0
         d.addCallback(lambda _: _setrev(cmd.rc))
         return d
@@ -402,7 +319,7 @@ class SVN(Source):
             if len(files) == 0:
                 d = defer.succeed(0)
             else:
-                if not self.slaveVersionIsOlderThan('rmdir', '2.14'):
+                if self.slaveVersionIsOlderThan('rmdir', '2.14'):
                     d = self.removeFiles(files)
                 else:
                     cmd = buildstep.RemoteCommand('rmdir', {'dir': files,
@@ -440,24 +357,23 @@ class SVN(Source):
                 continue
             yield filename
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def removeFiles(self, files):
         for filename in files:
             cmd = buildstep.RemoteCommand('rmdir', {'dir': filename,
                                                     'logEnviron': self.logEnviron,})
             cmd.useLog(self.stdio_log, False)
-            wfd = defer.waitForDeferred(self.runCommand(cmd))
-            yield wfd
-            wfd.getResult()
+            yield self.runCommand(cmd)
             if cmd.rc != 0:
-                yield cmd.rc
+                defer.returnValue(cmd.rc)
                 return
-        yield 0
+        defer.returnValue(0)
 
     def checkSvn(self):
         cmd = buildstep.RemoteShellCommand(self.workdir, ['svn', '--version'],
                                            env=self.env,
-                                           logEnviron=self.logEnviron)
+                                           logEnviron=self.logEnviron,
+                                           timeout=self.timeout)
         cmd.useLog(self.stdio_log, False)
         d = self.runCommand(cmd)
         def evaluate(cmd):
@@ -472,4 +388,4 @@ class SVN(Source):
             return None
         lastChange = max([int(c.revision) for c in changes])
         return lastChange
-    
+
