@@ -18,11 +18,7 @@ import os
 
 # apply the same patches the buildmaster does when it starts
 from buildbot import monkeypatches
-monkeypatches.patch_all()
-
-# and some extras to aid debugging
-from buildbot.util import monkeypatches
-monkeypatches.add_debugging_monkeypatches()
+monkeypatches.patch_all(for_tests=True)
 
 # import mock so we bail out early if it's not installed
 try:
