@@ -13,32 +13,40 @@
 #
 # Copyright Buildbot Team Members
 
-import os
-import sys
+from __future__ import with_statement
+
 import mock
+import os
 import shutil
-import StringIO
+import cStringIO as StringIO
+import sys
 
 import twisted
-from twisted.trial import unittest
 from twisted.internet import defer
-
+from twisted.trial import unittest
+from twisted.protocols import basic
 from buildbot.schedulers import trysched
-from buildbot.test.util import scheduler, dirs
+from buildbot.test.util import dirs
+from buildbot.test.util import scheduler
+from buildbot.util import json
+
 
 class TryBase(unittest.TestCase):
 
     def test_filterBuilderList_ok(self):
-        sched = trysched.TryBase(name='tsched', builderNames=['a', 'b', 'c'], properties={})
-        self.assertEqual(sched.filterBuilderList(['b', 'c']), [ 'b', 'c' ])
+        sched = trysched.TryBase(
+            name='tsched', builderNames=['a', 'b', 'c'], properties={})
+        self.assertEqual(sched.filterBuilderList(['b', 'c']), ['b', 'c'])
 
     def test_filterBuilderList_bad(self):
-        sched = trysched.TryBase(name='tsched', builderNames=['a', 'b'], properties={})
-        self.assertEqual(sched.filterBuilderList(['b', 'c']), [ ])
+        sched = trysched.TryBase(
+            name='tsched', builderNames=['a', 'b'], properties={})
+        self.assertEqual(sched.filterBuilderList(['b', 'c']), [])
 
     def test_filterBuilderList_empty(self):
-        sched = trysched.TryBase(name='tsched', builderNames=['a', 'b'], properties={})
-        self.assertEqual(sched.filterBuilderList([]), [ 'a', 'b' ])
+        sched = trysched.TryBase(
+            name='tsched', builderNames=['a', 'b'], properties={})
+        self.assertEqual(sched.filterBuilderList([]), ['a', 'b'])
 
 
 class JobdirService(dirs.DirsMixin, unittest.TestCase):
@@ -58,7 +66,8 @@ class JobdirService(dirs.DirsMixin, unittest.TestCase):
 
         # creat some new data to process
         jobdata = os.path.join(self.newdir, 'jobdata')
-        open(jobdata, "w").write('JOBDATA')
+        with open(jobdata, "w") as f:
+            f.write('JOBDATA')
 
         # stub out svc.parent.handleJobFile and .jobdir
         def handleJobFile(filename, f):
@@ -74,7 +83,7 @@ class JobdirService(dirs.DirsMixin, unittest.TestCase):
 
 class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
 
-    SCHEDULERID = 23
+    OBJECTID = 23
 
     def setUp(self):
         self.setUpScheduler()
@@ -92,12 +101,12 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         self.jobdir = os.path.abspath('jobdir')
         if os.path.exists(self.jobdir):
             shutil.rmtree(self.jobdir)
-        for subdir in 'new', 'cur':
-            os.makedirs(os.path.join(self.jobdir, subdir))
+        os.mkdir(self.jobdir)
 
         # build scheduler
         kwargs = dict(name="tsched", builderNames=['a'], jobdir=self.jobdir)
-        sched = self.attachScheduler(trysched.Try_Jobdir(**kwargs), self.SCHEDULERID)
+        sched = self.attachScheduler(
+            trysched.Try_Jobdir(**kwargs), self.OBJECTID)
 
         # start it
         sched.startService()
@@ -112,6 +121,11 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
                 'jobdir',
                 os.path.abspath('basedir/jobdir'))
 
+    def test_startService_reldir_subdir(self):
+        return self.do_test_startService(
+                'jobdir',
+                os.path.abspath('basedir/jobdir/cur'))
+
     def test_startService_absdir(self):
         return self.do_test_startService(
                 os.path.abspath('jobdir'),
@@ -120,26 +134,44 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
     # parseJob
 
     def test_parseJob_empty(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['a'], jobdir='foo')
+        self.assertRaises(
+            trysched.BadJobfile, sched.parseJob, StringIO.StringIO(''))
+
+    def test_parseJob_longer_than_netstring_MAXLENGTH(self):
+        self.patch(basic.NetstringReceiver, 'MAX_LENGTH', 100)
         sched = trysched.Try_Jobdir(name='tsched', builderNames=['a'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '1', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
+            'buildera', 'builderc'
+        )
+        jobstr += 'x' * 200
+
+        test_temp_file = StringIO.StringIO(jobstr)
+
         self.assertRaises(trysched.BadJobfile,
-            lambda : sched.parseJob(StringIO.StringIO('')))
+            lambda : sched.parseJob(test_temp_file))
 
     def test_parseJob_invalid(self):
-        sched = trysched.Try_Jobdir(name='tsched', builderNames=['a'], jobdir='foo')
-        self.assertRaises(trysched.BadJobfile,
-            lambda : sched.parseJob(StringIO.StringIO('this is not a netstring')))
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['a'], jobdir='foo')
+        self.assertRaises(
+            trysched.BadJobfile, sched.parseJob,
+            StringIO.StringIO('this is not a netstring'))
 
     def test_parseJob_invalid_version(self):
-        sched = trysched.Try_Jobdir(name='tsched', builderNames=['a'], jobdir='foo')
-        self.assertRaises(trysched.BadJobfile,
-            lambda : sched.parseJob(StringIO.StringIO('1:9,')))
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['a'], jobdir='foo')
+        self.assertRaises(
+            trysched.BadJobfile, sched.parseJob, StringIO.StringIO('1:9,'))
 
     def makeNetstring(self, *strings):
-        return ''.join([ '%d:%s,' % (len(s), s) for s in strings ])
+        return ''.join(['%d:%s,' % (len(s), s) for s in strings])
 
     def test_parseJob_v1(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
             '1', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'buildera', 'builderc'
@@ -155,12 +187,13 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             'project': '',
             'who': '',
             'comment': '',
-            'repository': ''
+            'repository': '',
+            'properties': {},
         })
 
     def test_parseJob_v1_empty_branch_rev(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
                 # blank branch, rev are turned to None
             '1', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
@@ -170,9 +203,27 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         self.assertEqual(parsedjob['branch'], None)
         self.assertEqual(parsedjob['baserev'], None)
 
+    def test_parseJob_v1_no_builders(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '1', 'extid', '', '', '1', 'this is my diff, -- ++, etc.'
+        )
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob['builderNames'], [])
+
+    def test_parseJob_v1_no_properties(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '1', 'extid', '', '', '1', 'this is my diff, -- ++, etc.'
+        )
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob['properties'], {})
+
     def test_parseJob_v2(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
             '2', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj',
@@ -189,12 +240,13 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             'project': 'proj',
             'who': '',
             'comment': '',
-            'repository': 'repo'
+            'repository': 'repo',
+            'properties': {},
         })
 
     def test_parseJob_v2_empty_branch_rev(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
                 # blank branch, rev are turned to None
             '2', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
@@ -206,8 +258,8 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         self.assertEqual(parsedjob['baserev'], None)
 
     def test_parseJob_v2_no_builders(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
             '2', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj',
@@ -215,9 +267,19 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
         self.assertEqual(parsedjob['builderNames'], [])
 
+    def test_parseJob_v2_no_properties(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '2', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
+            'repo', 'proj',
+        )
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob['properties'], {})
+
     def test_parseJob_v3(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
             '3', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who',
@@ -234,12 +296,13 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             'project': 'proj',
             'who': 'who',
             'comment': '',
-            'repository': 'repo'
+            'repository': 'repo',
+            'properties': {},
         })
 
     def test_parseJob_v3_empty_branch_rev(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
                 # blank branch, rev are turned to None
             '3', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
@@ -251,8 +314,8 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         self.assertEqual(parsedjob['baserev'], None)
 
     def test_parseJob_v3_no_builders(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
             '3', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who'
@@ -260,12 +323,22 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
         self.assertEqual(parsedjob['builderNames'], [])
 
+    def test_parseJob_v3_no_properties(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '3', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
+            'repo', 'proj', 'who'
+        )
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob['properties'], {})
+
     def test_parseJob_v4(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
             '4', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
-            'repo', 'proj', 'who', 'comment', 
+            'repo', 'proj', 'who', 'comment',
             'buildera', 'builderc'
         )
         parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
@@ -279,12 +352,13 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             'project': 'proj',
             'who': 'who',
             'comment': 'comment',
-            'repository': 'repo'
+            'repository': 'repo',
+            'properties': {},
         })
 
     def test_parseJob_v4_empty_branch_rev(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
                 # blank branch, rev are turned to None
             '4', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
@@ -296,8 +370,8 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         self.assertEqual(parsedjob['baserev'], None)
 
     def test_parseJob_v4_no_builders(self):
-        sched = trysched.Try_Jobdir(name='tsched',
-                builderNames=['buildera','builderb'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
             '4', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who', 'comment'
@@ -305,19 +379,106 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
         self.assertEqual(parsedjob['builderNames'], [])
 
+    def test_parseJob_v4_no_properties(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '4', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
+            'repo', 'proj', 'who', 'comment'
+        )
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob['properties'], {})
+
+    def test_parseJob_v5(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '5',
+            json.dumps({
+                'jobid': 'extid', 'branch': 'trunk', 'baserev': '1234',
+                'patch_level': 1, 'patch_body': 'this is my diff, -- ++, etc.',
+                'repository': 'repo', 'project': 'proj', 'who': 'who',
+                'comment': 'comment', 'builderNames': ['buildera', 'builderc'],
+                'properties': {'foo': 'bar'},
+            }))
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob, {
+            'baserev': '1234',
+            'branch': 'trunk',
+            'builderNames': ['buildera', 'builderc'],
+            'jobid': 'extid',
+            'patch_body': 'this is my diff, -- ++, etc.',
+            'patch_level': 1,
+            'project': 'proj',
+            'who': 'who',
+            'comment': 'comment',
+            'repository': 'repo',
+            'properties': {'foo': 'bar'},
+        })
+
+    def test_parseJob_v5_empty_branch_rev(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+                # blank branch, rev are turned to None
+            '4', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
+            'repo', 'proj', 'who', 'comment',
+            'buildera', 'builderc'
+        )
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob['branch'], None)
+        self.assertEqual(parsedjob['baserev'], None)
+
+    def test_parseJob_v5_no_builders(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '5',
+            json.dumps({
+                'jobid': 'extid', 'branch': 'trunk', 'baserev': '1234',
+                'patch_level': '1', 'diff': 'this is my diff, -- ++, etc.',
+                'repository': 'repo', 'project': 'proj', 'who': 'who',
+                'comment': 'comment', 'builderNames': [],
+                'properties': {'foo': 'bar'},
+            }))
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob['builderNames'], [])
+
+    def test_parseJob_v5_no_properties(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring(
+            '5',
+            json.dumps({
+                'jobid': 'extid', 'branch': 'trunk', 'baserev': '1234',
+                'patch_level': '1', 'diff': 'this is my diff, -- ++, etc.',
+                'repository': 'repo', 'project': 'proj', 'who': 'who',
+                'comment': 'comment', 'builderNames': ['buildera', 'builderb'],
+                'properties': {},
+            }))
+        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        self.assertEqual(parsedjob['properties'], {})
+
+    def test_parseJob_v5_invalid_json(self):
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
+        jobstr = self.makeNetstring('5', '{"comment": "com}')
+        self.assertRaises(
+            trysched.BadJobfile, sched.parseJob, StringIO.StringIO(jobstr))
+
     # handleJobFile
 
     def call_handleJobFile(self, parseJob):
         sched = self.attachScheduler(
-            trysched.Try_Jobdir(name='tsched', builderNames=['buildera','builderb'],
-                                jobdir='foo'),
-            self.SCHEDULERID)
-
+            trysched.Try_Jobdir(
+                name='tsched', builderNames=['buildera', 'builderb'],
+                jobdir='foo'), self.OBJECTID)
         fakefile = mock.Mock()
+
         def parseJob_(f):
             assert f is fakefile
             return parseJob(f)
-        sched.parseJob = parseJob
+        sched.parseJob = parseJob_
         return sched.handleJobFile('fakefile', fakefile)
 
     def makeSampleParsedJob(self, **overrides):
@@ -325,23 +486,28 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             builderNames=['buildera', 'builderb'],
             jobid='extid', patch_body='this is my diff, -- ++, etc.',
             patch_level=1, project='proj', repository='repo', who='who',
-            comment='comment')
+            comment='comment', properties={})
         pj.update(overrides)
         return pj
 
     def test_handleJobFile(self):
-        d = self.call_handleJobFile(lambda f : self.makeSampleParsedJob())
+        d = self.call_handleJobFile(lambda f: self.makeSampleParsedJob())
+
         def check(_):
             self.db.buildsets.assertBuildset('?',
                     dict(reason="'try' job by user who",
                         external_idstring='extid',
-                        properties=[('scheduler', ('tsched', 'Scheduler'))]),
-                    dict(branch='trunk', repository='repo',
+                        properties=[('scheduler', ('tsched', 'Scheduler'))],
+                        sourcestampsetid=100),
+                    {'':
+                     dict(branch='trunk', repository='repo',  codebase='',
                         project='proj', revision='1234',
                         patch_body='this is my diff, -- ++, etc.',
                         patch_level=1, patch_subdir='',
                         patch_author='who',
-                        patch_comment='comment'))
+                        patch_comment='comment',
+                        sourcestampsetid=100)
+                    })
         d.addCallback(check)
         return d
 
@@ -349,18 +515,22 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         def parseJob(f):
             raise trysched.BadJobfile
         d = self.call_handleJobFile(parseJob)
+
         def check(bsid):
             self.db.buildsets.assertBuildsets(0)
-            self.assertEqual(1, len(self.flushLoggedErrors(trysched.BadJobfile)))
+            self.assertEqual(
+                1, len(self.flushLoggedErrors(trysched.BadJobfile)))
         d.addCallback(check)
         return d
-    if twisted.version.major <= 9 and sys.version_info[:2] >= (2,7):
+    if twisted.version.major <= 9 and sys.version_info[:2] >= (2, 7):
         test_handleJobFile_exception.skip = (
-            "flushLoggedErrors does not work correctly on 9.0.0 and earlier with Python-2.7")
+            "flushLoggedErrors does not work correctly on 9.0.0 "
+            "and earlier with Python-2.7")
 
     def test_handleJobFile_bad_builders(self):
         d = self.call_handleJobFile(
-                lambda f : self.makeSampleParsedJob(builderNames=['xxx']))
+                lambda f: self.makeSampleParsedJob(builderNames=['xxx']))
+
         def check(_):
             self.db.buildsets.assertBuildsets(0)
         d.addCallback(check)
@@ -368,25 +538,60 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
 
     def test_handleJobFile_subset_builders(self):
         d = self.call_handleJobFile(
-                lambda f : self.makeSampleParsedJob(builderNames=['buildera']))
+            lambda f: self.makeSampleParsedJob(builderNames=['buildera']))
+
         def check(_):
             self.db.buildsets.assertBuildset('?',
                     dict(reason="'try' job by user who",
                         external_idstring='extid',
-                        properties=[('scheduler', ('tsched', 'Scheduler'))]),
-                    dict(branch='trunk', repository='repo',
+                        properties=[('scheduler', ('tsched', 'Scheduler'))],
+                        sourcestampsetid=100),
+                    {'':
+                     dict(branch='trunk', repository='repo',  codebase='',
                         project='proj', revision='1234',
                         patch_body='this is my diff, -- ++, etc.',
                         patch_level=1, patch_subdir='',
                         patch_author='who',
-                        patch_comment='comment'))
+                        patch_comment='comment',
+                        sourcestampsetid=100)
+                    })
         d.addCallback(check)
         return d
 
+    def test_handleJobFile_with_try_properties(self):
+        d = self.call_handleJobFile(
+            lambda f: self.makeSampleParsedJob(properties={'foo': 'bar'}))
+
+        def check(_):
+            self.db.buildsets.assertBuildset('?',
+                    dict(reason="'try' job by user who",
+                        external_idstring='extid',
+                        properties=[
+                            ('foo', ('bar', 'try build')),
+                            ('scheduler', ('tsched', 'Scheduler')),
+                        ],
+                        sourcestampsetid=100),
+                    {'':
+                     dict(branch='trunk', repository='repo',  codebase='',
+                        project='proj', revision='1234',
+                        patch_body='this is my diff, -- ++, etc.',
+                        patch_level=1, patch_subdir='',
+                        patch_author='who',
+                        patch_comment='comment',
+                        sourcestampsetid=100)
+                    })
+        d.addCallback(check)
+        return d
+
+    def test_handleJobFile_with_invalid_try_properties(self):
+        d = self.call_handleJobFile(
+            lambda f: self.makeSampleParsedJob(properties=['foo', 'bar']))
+        return self.assertFailure(d, AttributeError)
+
 
 class Try_Userpass_Perspective(scheduler.SchedulerMixin, unittest.TestCase):
 
-    SCHEDULERID = 26
+    OBJECTID = 26
 
     def setUp(self):
         self.setUpScheduler()
@@ -396,16 +601,12 @@ class Try_Userpass_Perspective(scheduler.SchedulerMixin, unittest.TestCase):
 
     def makeScheduler(self, **kwargs):
         sched = self.attachScheduler(trysched.Try_Userpass(**kwargs),
-                self.SCHEDULERID)
-
+                self.OBJECTID)
         # Try will return a remote version of master.status, so give it
         # something to return
         sched.master.status = mock.Mock()
-
         return sched
 
-    # tests
-
     def call_perspective_try(self, *args, **kwargs):
         sched = self.makeScheduler(name='tsched', builderNames=['a', 'b'],
                 port='xxx', userpass=[('a', 'b')], properties=dict(frm='schd'))
@@ -413,8 +614,10 @@ class Try_Userpass_Perspective(scheduler.SchedulerMixin, unittest.TestCase):
         return persp.perspective_try(*args, **kwargs)
 
     def test_perspective_try(self):
-        d = self.call_perspective_try('default', 'abcdef', (1, '-- ++'), 'repo',
-                'proj', ['a'], properties={'pr':'op'})
+        d = self.call_perspective_try(
+            'default', 'abcdef', (1, '-- ++'), 'repo', 'proj', ['a'],
+            properties={'pr': 'op'})
+
         def check(_):
             self.db.buildsets.assertBuildset('?',
                     dict(reason="'try' job",
@@ -423,17 +626,24 @@ class Try_Userpass_Perspective(scheduler.SchedulerMixin, unittest.TestCase):
                             ('frm', ('schd', 'Scheduler')),
                             ('pr', ('op', 'try build')),
                             ('scheduler', ('tsched', 'Scheduler')),
-                        ]),
-                    dict(branch='default', repository='repo',
+                        ],
+                        sourcestampsetid=100,
+                        ),
+                    {'':
+                     dict(branch='default', repository='repo', codebase='',
                         project='proj', revision='abcdef',
+                        sourcestampsetid=100,
                         patch_body='-- ++', patch_level=1, patch_subdir='',
-                        patch_author="", patch_comment=""))
+                        patch_author="", patch_comment="")
+                    })
         d.addCallback(check)
         return d
 
     def test_perspective_try_who(self):
-        d = self.call_perspective_try('default', 'abcdef', (1, '-- ++'), 'repo',
-                'proj', ['a'], who='who', comment='comment', properties={'pr':'op'})
+        d = self.call_perspective_try(
+            'default', 'abcdef', (1, '-- ++'), 'repo', 'proj', ['a'],
+            who='who', comment='comment', properties={'pr': 'op'})
+
         def check(_):
             self.db.buildsets.assertBuildset('?',
                     dict(reason="'try' job by user who (comment)",
@@ -442,17 +652,24 @@ class Try_Userpass_Perspective(scheduler.SchedulerMixin, unittest.TestCase):
                             ('frm', ('schd', 'Scheduler')),
                             ('pr', ('op', 'try build')),
                             ('scheduler', ('tsched', 'Scheduler')),
-                        ]),
-                    dict(branch='default', repository='repo',
+                        ],
+                        sourcestampsetid=100,
+                        ),
+                    {'':
+                     dict(branch='default', repository='repo',  codebase='',
                         project='proj', revision='abcdef',
+                        sourcestampsetid=100,
                         patch_body='-- ++', patch_level=1, patch_subdir='',
-                        patch_author='who', patch_comment="comment"))
+                        patch_author='who', patch_comment="comment")
+                    })
         d.addCallback(check)
         return d
 
     def test_perspective_try_bad_builders(self):
-        d = self.call_perspective_try('default', 'abcdef', (1, '-- ++'), 'repo',
-                'proj', ['xxx'], properties={'pr':'op'})
+        d = self.call_perspective_try(
+            'default', 'abcdef', (1, '-- ++'), 'repo', 'proj', ['xxx'],
+            properties={'pr': 'op'})
+
         def check(_):
             self.db.buildsets.assertBuildsets(0)
         d.addCallback(check)
@@ -462,16 +679,18 @@ class Try_Userpass_Perspective(scheduler.SchedulerMixin, unittest.TestCase):
         sched = self.makeScheduler(name='tsched', builderNames=['a', 'b'],
                 port='xxx', userpass=[('a', 'b')])
         persp = trysched.Try_Userpass_Perspective(sched, 'a')
-        d = defer.maybeDeferred(lambda :
-                persp.perspective_getAvailableBuilderNames())
+        d = defer.maybeDeferred(
+            lambda: persp.perspective_getAvailableBuilderNames())
+
         def check(buildernames):
             self.assertEqual(buildernames, ['a', 'b'])
         d.addCallback(check)
         return d
 
+
 class Try_Userpass(scheduler.SchedulerMixin, unittest.TestCase):
 
-    SCHEDULERID = 25
+    OBJECTID = 25
 
     def setUp(self):
         self.setUpScheduler()
@@ -481,33 +700,28 @@ class Try_Userpass(scheduler.SchedulerMixin, unittest.TestCase):
 
     def makeScheduler(self, **kwargs):
         sched = self.attachScheduler(trysched.Try_Userpass(**kwargs),
-                self.SCHEDULERID)
+                self.OBJECTID)
         return sched
 
-    # tests
-
     def test_service(self):
         sched = self.makeScheduler(name='tsched', builderNames=['a'],
                 port='tcp:9999', userpass=[('fred', 'derf')])
-
         # patch out the pbmanager's 'register' command both to be sure
         # the registration is correct and to get a copy of the factory
         registration = mock.Mock()
-        registration.unregister = lambda : defer.succeed(None)
+        registration.unregister = lambda: defer.succeed(None)
         sched.master.pbmanager = mock.Mock()
+
         def register(portstr, user, passwd, factory):
             self.assertEqual([portstr, user, passwd],
                              ['tcp:9999', 'fred', 'derf'])
             self.got_factory = factory
             return registration
         sched.master.pbmanager.register = register
-
         # start it
         sched.startService()
-
         # make a fake connection by invoking the factory, and check that we
         # get the correct perspective
         persp = self.got_factory(mock.Mock(), 'fred')
         self.failUnless(isinstance(persp, trysched.Try_Userpass_Perspective))
-
         return sched.stopService()
