@@ -13,27 +13,32 @@
 #
 # Copyright Buildbot Team Members
 
+from zope.interface import implements
+
 from twisted.trial import unittest
 from twisted.python import failure
 from twisted.internet import defer, reactor
-from buildbot import config
+from buildbot import config, interfaces
 from buildbot.process import properties
-from buildbot.schedulers import triggerable
 from buildbot.status import master
 from buildbot.status.results import SUCCESS, FAILURE, EXCEPTION
 from buildbot.steps import trigger
 from buildbot.test.util import steps, compat
 from buildbot.test.fake import fakemaster, fakedb
 
-class FakeTriggerable(triggerable.Triggerable):
+class FakeTriggerable(object):
+    implements(interfaces.ITriggerableScheduler)
 
     triggered_with = None
     result = SUCCESS
     brids = {}
     exception = False
 
-    def trigger(self, sourcestampsetid, set_props=None):
-        self.triggered_with = (sourcestampsetid, set_props.properties)
+    def __init__(self, name):
+        self.name = name
+
+    def trigger(self, sourcestamps = None, set_props=None):
+        self.triggered_with = (sourcestamps, set_props.properties)
         d = defer.Deferred()
         if self.exception:
             reactor.callLater(0, d.errback, RuntimeError('oh noes'))
@@ -44,16 +49,11 @@ class FakeTriggerable(triggerable.Triggerable):
 
 class FakeSourceStamp(object):
 
-    def __init__(self, _ssid, _setid, **kwargs):
-        self._ssid = _ssid
-        self._setid = _setid
+    def __init__(self, **kwargs):
         self.__dict__.update(kwargs)
 
-    def getAbsoluteSourceStamp(self, rev):
-        return FakeSourceStamp(self._ssid + 1000, self._setid + 1000, revision=rev)
-
-    def getSourceStampSetId(self, master):
-        return defer.succeed(self._setid)
+    def asDict(self, includePatch = True):
+        return self.__dict__.copy()
 
 # Magic numbers that relate brid to other build settings
 BRID_TO_BSID = lambda brid: brid+2000
@@ -62,32 +62,30 @@ BRID_TO_BUILD_NUMBER = lambda brid: brid+4000
 
 class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
 
-    THIS_SSID = 6
-    THIS_SS_SETID = 66
-    ABS_SS_SETID = THIS_SS_SETID + 1000
-    NEW_SSID = 987
-    NEW_SS_SETID = 800
-    
     def setUp(self):
         return self.setUpBuildStep()
 
     def tearDown(self):
         return self.tearDownBuildStep()
 
-    def setupStep(self, *args, **kwargs):
-        steps.BuildStepMixin.setupStep(self, *args, **kwargs)
+    def setupStep(self, step, sourcestampsInBuild=None, gotRevisionsInBuild=None, *args, **kwargs):
+        sourcestamps = sourcestampsInBuild or []
+        got_revisions = gotRevisionsInBuild or {}
+
+        steps.BuildStepMixin.setupStep(self, step, *args, **kwargs)
 
         # This step reaches deeply into a number of parts of Buildbot.  That
         # should be fixed!
 
         # set up a buildmaster that knows about two fake schedulers, a and b
-        self.build.builder.botmaster.parent = m = fakemaster.make_master()
+        m = fakemaster.make_master()
+        self.build.builder.botmaster = m.botmaster
         m.db = fakedb.FakeDBConnector(self)
         m.status = master.Status(m)
         m.config.buildbotURL = "baseurl/"
 
-        self.scheduler_a = a = FakeTriggerable(name='a', builderNames=['A'])
-        self.scheduler_b = b = FakeTriggerable(name='b', builderNames=['B'])
+        self.scheduler_a = a = FakeTriggerable(name='a')
+        self.scheduler_b = b = FakeTriggerable(name='b')
         def allSchedulers():
             return [ a, b ]
         m.allSchedulers = allSchedulers
@@ -101,7 +99,7 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
         make_fake_build = lambda brid: fakedb.Build(brid=brid,
                                                     id=BRID_TO_BID(brid),
                                                     number=BRID_TO_BUILD_NUMBER(brid))
-        
+
         m.db.insertTestData([
                make_fake_br(11, "A"),
                make_fake_br(22, "B"),
@@ -109,22 +107,13 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
                make_fake_build(22),
         ])
 
-        # the build has a getSourceStamp method
-        def getSourceStamp():
-            return FakeSourceStamp(self.THIS_SSID, self.THIS_SS_SETID)
-        self.build.getSourceStamp = getSourceStamp
+        def getAllSourceStamps():
+            return sourcestamps
+        self.build.getAllSourceStamps = getAllSourceStamps
+        def getAllGotRevisions():
+            return got_revisions
+        self.build.build_status.getAllGotRevisions = getAllGotRevisions
 
-        # the db has a addSourceStamp method
-        def addSourceStamp(**kwargs):
-            self.addSourceStamp_kwargs = kwargs
-            return defer.succeed(self.NEW_SSID)
-        m.db.sourcestamps.addSourceStamp = addSourceStamp
-
-        # the db has a addSourceStampSet method
-        def addSourceStampSet():
-            return defer.succeed(self.NEW_SS_SETID)
-        m.db.sourcestampsets.addSourceStampSet = addSourceStampSet
-        
         self.exp_add_sourcestamp = None
         self.exp_a_trigger = None
         self.exp_b_trigger = None
@@ -173,11 +162,11 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
     def expectTriggeredLinks(self, *args):
         def get_args(sch, name):
             label = lambda name, num: "%s #%d" % (name, num)
-            url = lambda name, num: "baseurl/builders/%s/builds/%d" % (name, num ) 
+            url = lambda name, num: "baseurl/builders/%s/builds/%d" % (name, num )
 
             num = BRID_TO_BUILD_NUMBER(sch.brids[name])
 
-            #returns the *args and **kwargs that will be called on addURL... 
+            #returns the *args and **kwargs that will be called on addURL...
             #   which is just addURL('label', 'url')
             return ( (label(name,num), url(name,num)) , {} )
 
@@ -198,6 +187,12 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
                 trigger.Trigger(schedulerNames=['c'],
                     sourceStamp=dict(x=1), updateSourceStamp=True))
 
+    def test_sourceStamps_and_updateSourceStamp(self):
+        self.assertRaises(config.ConfigErrors, lambda :
+                trigger.Trigger(schedulerNames=['c'],
+                    sourceStamps=[dict(x=1), dict(x=2)],
+                    updateSourceStamp=True))
+
     def test_updateSourceStamp_and_alwaysUseLatest(self):
         self.assertRaises(config.ConfigErrors, lambda :
                 trigger.Trigger(schedulerNames=['c'],
@@ -208,10 +203,16 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
                 trigger.Trigger(schedulerNames=['c'],
                     sourceStamp=dict(x=1), alwaysUseLatest=True))
 
+    def test_sourceStamps_and_alwaysUseLatest(self):
+        self.assertRaises(config.ConfigErrors, lambda :
+                trigger.Trigger(schedulerNames=['c'],
+                    sourceStamps=[dict(x=1), dict(x=2)],
+                    alwaysUseLatest=True))
+
     def test_simple(self):
-        self.setupStep(trigger.Trigger(schedulerNames=['a']))
+        self.setupStep(trigger.Trigger(schedulerNames=['a'], sourceStamps = {}))
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID, {}))
+        self.expectTriggeredWith(a=({}, {}))
         return self.runStep()
 
     def test_simple_failure(self):
@@ -220,7 +221,7 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
         # not waitForFinish, so trigger step succeeds even though the build
         # didn't fail
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID, {}))
+        self.expectTriggeredWith(a=({}, {}))
         return self.runStep()
 
     @compat.usesFlushLoggedErrors
@@ -228,7 +229,7 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
         self.setupStep(trigger.Trigger(schedulerNames=['a']))
         self.scheduler_a.exception = True
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID, {}))
+        self.expectTriggeredWith(a=( {}, {}))
         d = self.runStep()
         def flush(_):
             self.assertEqual(len(self.flushLoggedErrors(RuntimeError)), 1)
@@ -237,69 +238,189 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
 
     def test_bogus_scheduler(self):
         self.setupStep(trigger.Trigger(schedulerNames=['a', 'x']))
-        self.expectOutcome(result=FAILURE, status_text=['no scheduler:', 'x'])
+        self.expectOutcome(result=FAILURE, status_text=['not valid scheduler:', 'x'])
         self.expectTriggeredWith(a=None) # a is not triggered!
         return self.runStep()
 
     def test_updateSourceStamp(self):
         self.setupStep(trigger.Trigger(schedulerNames=['a'],
-            updateSourceStamp=True))
-        self.properties.setProperty('got_revision', 23456, 'Source')
+                                       updateSourceStamp=True),
+                       sourcestampsInBuild = [FakeSourceStamp(codebase='',
+                                                              repository='x',
+                                                              revision=11111)
+                                              ],
+                       gotRevisionsInBuild = {'': 23456},
+                       )
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.ABS_SS_SETID, {}))
+        self.expectTriggeredWith(a=({'':{'codebase':'',
+                                         'repository': 'x',
+                                         'revision': 23456}
+                                    }, {}))
         return self.runStep()
 
-    def test_updateSourceStamp_no_prop(self):
+    def test_updateSourceStamp_no_got_revision(self):
         self.setupStep(trigger.Trigger(schedulerNames=['a'],
-            updateSourceStamp=True))
+                                       updateSourceStamp=True),
+                       sourcestampsInBuild = [FakeSourceStamp(codebase='',
+                                                              repository='x',
+                                                              revision=11111)
+                                              ])
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID, {}))
+        self.expectTriggeredWith(a=({'':{'codebase':'',
+                                         'repository': 'x',
+                                         'revision': 11111} # uses old revision
+                                    }, {}))
         return self.runStep()
 
-    def test_updateSourceStamp_prop(self):
+    def test_not_updateSourceStamp(self):
         self.setupStep(trigger.Trigger(schedulerNames=['a'],
-            updateSourceStamp=properties.Property('usess')))
-        self.properties.setProperty('got_revision', 23456, 'Source')
+                                       updateSourceStamp=False),
+                       sourcestampsInBuild = [FakeSourceStamp(codebase='',
+                                                              repository='x',
+                                                              revision=11111)
+                                              ],
+                       gotRevisionsInBuild = {'': 23456},
+                       )
+        self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
+        self.expectTriggeredWith(a=({'':{'codebase':'',
+                                          'repository': 'x',
+                                          'revision': 11111}
+                                    }, {}))
+        return self.runStep()
+
+    def test_updateSourceStamp_multiple_repositories(self):
+        self.setupStep(trigger.Trigger(schedulerNames=['a'],
+                                       updateSourceStamp=True),
+                       sourcestampsInBuild = [
+                                              FakeSourceStamp(codebase='cb1',
+                                                              revision='12345'),
+                                              FakeSourceStamp(codebase='cb2',
+                                                              revision='12345')
+                                              ],
+                       gotRevisionsInBuild = {'cb1': 23456, 'cb2': 34567},
+                       )
+        self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
+        self.expectTriggeredWith(a=({'cb1': {'codebase':'cb1',
+                                             'revision':23456},
+                                     'cb2': {'codebase':'cb2',
+                                             'revision':34567}
+                                    }, {}))
+        return self.runStep()
+
+    def test_updateSourceStamp_prop_false(self):
+        self.setupStep(trigger.Trigger(schedulerNames=['a'],
+                               updateSourceStamp=properties.Property('usess')),
+                       sourcestampsInBuild = [FakeSourceStamp(codebase='',
+                                                              repository='x',
+                                                              revision=11111)
+                                             ],
+                       gotRevisionsInBuild = {'': 23456},
+                       )
         self.properties.setProperty('usess', False, 'me')
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        # didn't use SS
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID, {}))
+        # didn't use got_revision
+        self.expectTriggeredWith(a=({'': { 'codebase':'',
+                                           'repository': 'x',
+                                           'revision': 11111
+                                    }}, {}))
+        return self.runStep()
+
+    def test_updateSourceStamp_prop_true(self):
+        self.setupStep(trigger.Trigger(schedulerNames=['a'],
+                               updateSourceStamp=properties.Property('usess')),
+                       sourcestampsInBuild = [FakeSourceStamp(codebase='',
+                                                              repository='x',
+                                                              revision=11111)
+                                             ],
+                       gotRevisionsInBuild = {'': 23456},
+                       )
+        self.properties.setProperty('usess', True, 'me')
+        self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
+        # didn't use got_revision
+        self.expectTriggeredWith(a=({'': { 'codebase':'',
+                                           'repository': 'x',
+                                           'revision': 23456
+                                    }}, {}))
         return self.runStep()
 
     def test_alwaysUseLatest(self):
         self.setupStep(trigger.Trigger(schedulerNames=['b'],
-            alwaysUseLatest=True))
+                                       alwaysUseLatest=True),
+                       sourcestampsInBuild = [FakeSourceStamp(codebase='',
+                                                              repository='x',
+                                                              revision=11111)
+                                             ])
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'b'])
-        self.expectTriggeredWith(b=(None, {}))
+        # Do not pass setid
+        self.expectTriggeredWith(b=({}, {}))
         return self.runStep()
 
-    def test_alwaysUseLatest_prop(self):
+    def test_alwaysUseLatest_prop_false(self):
         self.setupStep(trigger.Trigger(schedulerNames=['b'],
-            alwaysUseLatest=properties.Property('aul')))
+                                       alwaysUseLatest=properties.Property('aul')),
+                       sourcestampsInBuild = [FakeSourceStamp(codebase='',
+                                                              repository='x',
+                                                              revision=11111)
+                                              ])
         self.properties.setProperty('aul', False, 'me')
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'b'])
         # didn't use latest
-        self.expectTriggeredWith(b=(self.THIS_SS_SETID, {}))
+        self.expectTriggeredWith(b=({'': { 'codebase':'',
+                                           'repository': 'x',
+                                           'revision': 11111}
+                                    }, {}))
+        return self.runStep()
+
+    def test_alwaysUseLatest_prop_true(self):
+        self.setupStep(trigger.Trigger(schedulerNames=['b'],
+                                       alwaysUseLatest=properties.Property('aul')),
+                       sourcestampsInBuild = [FakeSourceStamp(codebase='',
+                                                              repository='x',
+                                                              revision=11111)
+                                              ])
+        self.properties.setProperty('aul', True, 'me')
+        self.expectOutcome(result=SUCCESS, status_text=['triggered', 'b'])
+        # didn't use latest
+        self.expectTriggeredWith(b=({}, {}))
         return self.runStep()
 
     def test_sourceStamp(self):
+        ss = dict(revision=9876, branch='dev')
         self.setupStep(trigger.Trigger(schedulerNames=['b'],
-            sourceStamp=dict(revision=9876, branch='dev')))
+            sourceStamp=ss))
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'b'])
-        self.expectAddedSourceStamp(revision=9876, branch='dev', 
-                                    sourcestampsetid=self.NEW_SS_SETID)
-        self.expectTriggeredWith(b=(self.NEW_SS_SETID, {}))
+        self.expectTriggeredWith(b=({'': ss}, {}))
         return self.runStep()
 
+    def test_set_of_sourceStamps(self):
+        ss1 = dict(codebase='cb1', repository='r1', revision=9876, branch='dev')
+        ss2 = dict(codebase='cb2',repository='r2', revision=5432, branch='dev')
+        self.setupStep(trigger.Trigger(schedulerNames=['b'],
+            sourceStamps=[ss1,ss2]))
+        self.expectOutcome(result=SUCCESS, status_text=['triggered', 'b'])
+        self.expectTriggeredWith(b=({'cb1':ss1, 'cb2':ss2}, {}))
+        return self.runStep()
+
+    def test_set_of_sourceStamps_override_build(self):
+        ss1 = dict(codebase='cb1', repository='r1', revision=9876, branch='dev')
+        ss2 = dict(codebase='cb2',repository='r2', revision=5432, branch='dev')
+        ss3 = FakeSourceStamp(codebase='cb3', repository='r3', revision=1234, branch='dev')
+        ss4 = FakeSourceStamp(codebase='cb4',repository='r4', revision=2345, branch='dev')
+        self.setupStep(trigger.Trigger(schedulerNames=['b'],
+            sourceStamps=[ss1,ss2]), sourcestampsInBuild=[ss3, ss4])
+        self.expectOutcome(result=SUCCESS, status_text=['triggered', 'b'])
+        self.expectTriggeredWith(b=({'cb1':ss1, 'cb2':ss2}, {}))
+        return self.runStep()
+
+
     def test_sourceStamp_prop(self):
         self.setupStep(trigger.Trigger(schedulerNames=['b'],
             sourceStamp=dict(revision=properties.Property('rev'),
                 branch='dev')))
         self.properties.setProperty('rev', 602, 'me')
+        expected_ss = dict(revision=602, branch='dev')
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'b'])
-        self.expectAddedSourceStamp(revision=602, branch='dev',
-                                    sourcestampsetid=self.NEW_SS_SETID)
-        self.expectTriggeredWith(b=(self.NEW_SS_SETID, {}))
+        self.expectTriggeredWith(b=({'': expected_ss}, {}))
         return self.runStep()
 
     def test_waitForFinish(self):
@@ -307,8 +428,8 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
             waitForFinish=True))
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a', 'b'])
         self.expectTriggeredWith(
-            a=(self.THIS_SS_SETID, {}),
-            b=(self.THIS_SS_SETID, {}))
+            a=({}, {}),
+            b=({}, {}))
         self.expectTriggeredLinks('a','b')
         return self.runStep(expect_waitForFinish=True)
 
@@ -317,7 +438,7 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
             waitForFinish=True))
         self.scheduler_a.result = FAILURE
         self.expectOutcome(result=FAILURE, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID, {}))
+        self.expectTriggeredWith(a=({}, {}))
         self.expectTriggeredLinks('a')
         return self.runStep(expect_waitForFinish=True)
 
@@ -329,8 +450,8 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
         self.expectOutcome(result=EXCEPTION,
                         status_text=['triggered', 'a', 'b'])
         self.expectTriggeredWith(
-            a=(self.THIS_SS_SETID, {}),
-            b=(self.THIS_SS_SETID, {}))
+            a=({}, {}),
+            b=({}, {}))
         self.expectTriggeredLinks('a') # b doesnt return a brid
         d = self.runStep(expect_waitForFinish=True)
         def flush(_):
@@ -342,7 +463,7 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
         self.setupStep(trigger.Trigger(schedulerNames=['a'],
                 set_properties=dict(x=1, y=2)))
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID,
+        self.expectTriggeredWith(a=({},
                                 dict(x=(1, 'Trigger'), y=(2, 'Trigger'))))
         return self.runStep()
 
@@ -351,7 +472,7 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
                 set_properties=dict(x=properties.Property('X'), y=2)))
         self.properties.setProperty('X', 'xxx', 'here')
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID,
+        self.expectTriggeredWith(a=({},
                                 dict(x=('xxx', 'Trigger'), y=(2, 'Trigger'))))
         return self.runStep()
 
@@ -362,16 +483,16 @@ class TestTrigger(steps.BuildStepMixin, unittest.TestCase):
         self.properties.setProperty('b', 'B', 'BB')
         self.properties.setProperty('c', 'C', 'CC')
         self.expectOutcome(result=SUCCESS, status_text=['triggered', 'a'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID,
-                            dict(a=('A', 'AA (in triggering build)'),
-                                 b=('B', 'BB (in triggering build)'))))
+        self.expectTriggeredWith(a=({},
+                            dict(a=('A', 'Trigger'),
+                                 b=('B', 'Trigger'))))
         return self.runStep()
 
     def test_interrupt(self):
         self.setupStep(trigger.Trigger(schedulerNames=['a'],
             waitForFinish=True))
         self.expectOutcome(result=EXCEPTION, status_text=['interrupted'])
-        self.expectTriggeredWith(a=(self.THIS_SS_SETID, {}))
+        self.expectTriggeredWith(a=({}, {}))
         d = self.runStep(expect_waitForFinish=True)
 
         # interrupt before the callLater representing the Triggerable
