@@ -20,9 +20,6 @@ from sqlalchemy.sql.expression import Executable, ClauseElement
 # from http://www.sqlalchemy.org/docs/core/compiler.html#compiling-sub-elements-of-a-custom-expression-construct
 
 class InsertFromSelect(Executable, ClauseElement):
-    """
-    An L{Executable} that can insert into C{table} the values from C{select}
-    """
     def __init__(self, table, select):
         self.table = table
         self.select = select
